﻿namespace Phoenix.Model.Bll.Domain
{
    using System.Collections.Generic;
    using System.Drawing;

    /// <summary>
    /// It is a group of keyword. 
    /// </summary>
    public class KeywordGroup : Poco
    {
        #region Fields

        private const char KEYWORD_SEP = ' ';

        private List<string> keywords = new List<string>();
        string rawKeywords = string.Empty;

        #endregion Fields

        #region Constructors

        public KeywordGroup()
        {
            this.HtmlColor = ColorTranslator.ToHtml(Color.Black);
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the color of the keyword which belong to this group.
        /// </summary>
        /// <value>The color.</value>
        public virtual Color Color
        {
            get
            {
                return ColorTranslator.FromHtml(this.HtmlColor);
            }
            set
            {
                this.HtmlColor = ColorTranslator.ToHtml(value);
            }
        }

        /// <summary>
        /// Gets or sets the keywords which belong to this group.
        /// </summary>
        /// <value>The keywords.</value>
        public virtual IList<string> Keywords
        {
            get { return this.keywords; }
        }

        /// <summary>
        /// Gets or sets the raw keywords, that's space separated keywords.
        /// </summary>
        /// <value>The raw keywords.</value>
        public virtual string RawKeywords
        {
            get
            {
                return this.rawKeywords;
            }
            set
            {
                this.rawKeywords = value;
                this.keywords = new List<string>();
                foreach (var keyword in this.RawKeywords.Split(KEYWORD_SEP))
                {
                    if (keyword.Length > 1 && keyword[0] != KEYWORD_SEP)
                        this.keywords.Add(keyword.Trim());
                }

            }
        }

        /// <summary>
        /// Gets or sets remarks to explain the group.
        /// </summary>
        /// <value>The remark.</value>
        public virtual string Remark
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the title of the group.
        /// </summary>
        /// <value>The title.</value>
        public virtual string Title
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the color in HTML mode.
        /// </summary>
        /// <value>The color of the HTML.</value>
        protected virtual string HtmlColor
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            string rgbName = string.Format("R: {0}, G: {1}, B: {2}"
                , this.Color.R
                , this.Color.G
                , this.Color.B);

            return string.Format("{0} [{1}]"
                , this.Title
                , this.Color.IsNamedColor
                    ? this.Color.Name
                    : rgbName);
        }

        #endregion Methods
    }
}