﻿namespace Phoenix.Model.Bll.Interface
{
    using System;
    using System.Collections.Generic;

    public interface IRepository<T> : IDisposable
    {
        #region Methods

        /// <summary>
        /// Creates the specified item in the database.
        /// </summary>
        /// <param name="item">The item.</param>
        void Create(T item);

        /// <summary>
        /// Creates the specified items in the database and in one transaction.
        /// </summary>
        /// <param name="items">The items.</param>
        void Create(IList<T> items);

        /// <summary>
        /// Creates the or update the specified item agains the database.
        /// </summary>
        /// <param name="item">The item.</param>
        void CreateOrUpdate(T item);

        /// <summary>
        /// Creates the or update the specified items agains the database and in one transaction.
        /// </summary>
        /// <param name="items">The items.</param>
        void CreateOrUpdate(IList<T> items);

        /// <summary>
        /// Deletes the specified item from the database.
        /// </summary>
        /// <param name="item">The item.</param>
        void Delete(T item);

        /// <summary>
        /// Deletes the specified items from the database and in one transaction.
        /// </summary>
        /// <param name="items">The items.</param>
        void Delete(IList<T> items);

        /// <summary>
        /// Gets all the rows. Equivalent of a SELECT *
        /// </summary>
        /// <returns>All the rows of the repository</returns>
        List<T> GetAllRows();

        /// <summary>
        /// Gets all the rows. Equivalent of a SELECT *
        /// </summary>
        /// <param name="orderBy">The order by clause.</param>
        /// <returns>All the rows of the repository</returns>
        List<T> GetAllRows(OrderBy orderBy);

        /// <summary>
        /// Determines whether the specified item is referenced.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns>
        /// 	<c>true</c> if the specified item is referenced; otherwise, <c>false</c>.
        /// </returns>
        bool IsReferenced(T item);

        /// <summary>
        /// Orders the specified list of the patient.
        /// </summary>
        /// <param name="patient">The patient.</param>
        /// <param name="on">The list of item link to the patient this method has to order.</param>
        void Order(IList<T> list, string propertyName);

        /// <summary>
        /// Updates the specified item against the database.
        /// </summary>
        /// <param name="item">The item.</param>
        void Update(T item);

        /// <summary>
        /// Updates the specified items against the database and in one transaction.
        /// </summary>
        /// <param name="items">The items.</param>
        void Update(IList<T> items);

        #endregion Methods
    }
}