﻿namespace Phoenix.Model.Management
{
    using System;

    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal;
    using Phoenix.Presenter.Interfaces.Manager;

    /// <summary>
    /// Provides feature to manage medical card types.
    /// </summary>
    public class MedicalCardTypeModel : Manager<MedicalCardType>
    {
        #region Fields

        private IMedicalCardTypeManagerPresenter view;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="MedicalCardTypeModel"/> class.
        /// </summary>
        /// <param name="presenter">The view.</param>
        public MedicalCardTypeModel(IMedicalCardTypeManagerPresenter presenter)
            : this(presenter, RepositoryFactory.Get<IMedicalCardTypeRepository>())
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MedicalCardTypeModel"/> class.
        /// </summary>
        /// <param name="presenter">The view.</param>
        /// <param name="repository">The repository.</param>
        public MedicalCardTypeModel(IMedicalCardTypeManagerPresenter presenter, IRepository<MedicalCardType> repository)
            : base(repository)
        {
            this.view = presenter;
            this.view.Inserted += view_Added;
            this.view.Deleted += view_Deleted;
            this.view.Updated += view_Updated;
            this.view.Refreshed += (sender, e) => this.RefreshData();
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns></returns>
        public override bool ValidateData()
        {
            bool result = true;

            this.view.ClearError();

            if (string.IsNullOrEmpty(this.view.ItemToManage.Title))
            {
                result = false;
                this.view.Invalidate(MedicalCardTypeFailedOn.EmptyTitle,
                    "The type should have a title!");
            }
            return result;
        }

        /// <summary>
        /// Loads this instance.
        /// </summary>
        private void RefreshData()
        {
            this.view.MedicalCardTypes = this.Repository.GetAllRows();
        }

        private void view_Added(object sender, EventArgs e)
        {
            if (this.ValidateData())
            {
                this.Insert(this.view.ItemToManage);
                this.RefreshData();
            }
        }

        private void view_Deleted(object sender, EventArgs e)
        {
            if (this.ValidateData())
            {
                this.Delete(this.view.ItemToManage);
                this.RefreshData();
            }
        }

        private void view_Updated(object sender, EventArgs e)
        {
            if (this.ValidateData())
            {
                this.Update(this.view.ItemToManage);
                this.RefreshData();
            }
        }

        #endregion Methods
    }
}