﻿namespace Phoenix.Model.Search
{
    using System.Collections.Generic;

    using Phoenix.Contracts;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal;
    using Phoenix.Presenter.Interfaces.Interfaces;

    /// <summary>
    /// Provide a Searcher with features focused on a Doctor.
    /// </summary>   
    public class DoctorSearcher : Searcher<Doctor>
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DoctorSearcher"/> class.
        /// </summary>
        /// <param name="presenter">The presenter.</param>
        public DoctorSearcher(ISearchBox<Doctor> presenter)
            : base(presenter)
        {
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets all the the doctor types.
        /// </summary>
        /// <value>The doctor types.</value>
        public IList<DoctorType> DoctorTypes
        {
            get
            {
                return RepositoryFactory.Get<IDoctorTypeRepository>().GetAllRows();
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Deletes the specified item from the repository.
        /// </summary>
        /// <param name="item">The item.</param>
        public override void Delete(Doctor item)
        {
            Assert.IsTrue((item != null), "The value of the doctor to delete can't be 'null'!");
            RepositoryFactory.Get<IDoctorRepository>().Delete(item);
        }

        /// <summary>
        /// Start a search on a user demand.
        /// </summary>
        /// <param name="criterion">The criterion.</param>
        /// <returns></returns>
        public override IList<Doctor> ProcessSearch(object criterion)
        {
            Doctor doctor = criterion as Doctor;
            Assert.IsTrue(doctor != null, "The criterion should be a 'Doctor'");

            IList<Doctor> result = RepositoryFactory.Get<IDoctorRepository>().Search(doctor);
            if (result.Count == 0)
                this.View.WarnOnZeroResult();
            return result;
        }

        #endregion Methods
    }
}