﻿namespace Phoenix.Model.Dal.Repositories
{
    using System.Linq;

    using NHibernate.Linq;

    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;

    public class MedicalCardTypeRepository : Repository<MedicalCardType>, IMedicalCardTypeRepository
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="MedicalCardRepository"/> class.
        /// </summary>
        public MedicalCardTypeRepository()
            : base("MedicalCardType", "Title")
        {
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Determines whether the specified item is referenced.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns>
        /// 	<c>true</c> if the specified item is referenced; otherwise, <c>false</c>.
        /// </returns>
        public override bool IsReferenced(MedicalCardType item)
        {
            return (from Patient p in this.Session.Linq<Patient>()
                    select p.MedicalCards into medicalCards

                    from MedicalCard c in medicalCards
                    where c.Type == item
                    select c).Count() > 0;
        }

        #endregion Methods
    }
}