﻿namespace Phoenix.Model.Dal.Repositories
{
    using System;
    using System.Collections.Generic;
    using System.Linq;

    using NHibernate.Linq;

    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;

    public class MeetingRepository : Repository<Meeting>, IMeetingRepository
    {
        #region Constructors

        public MeetingRepository()
            : base("Meeting", "StartDate")
        {
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Gets the meetings for the specified day.
        /// </summary>
        /// <param name="date">The date.</param>
        /// <returns>The meetings.</returns>
        public IList<Meeting> GetMeetings(DateTime date)
        {
            var query = from meeting in this.Session.Linq<Meeting>()
                        where meeting.StartDate >= date.Date.Date
                        && meeting.StartDate < date.Date.AddDays(1).Date
                        && meeting.IsPersonal == true
                        select meeting;
            return query.ToList();
        }

        /// <summary>
        /// Gets the meetings between the specified dates.
        /// </summary>
        /// <param name="startDate"></param>
        /// <param name="endDate"></param>
        /// <returns>The meetings</returns>
        public IList<Meeting> GetMeetings(DateTime startDate, DateTime endDate)
        {
            return (from meeting in this.Session.Linq<Meeting>()
                    where meeting.StartDate >= startDate
                    && meeting.StartDate <= endDate.AddDays(1)
                    && meeting.IsPersonal == true
                    select meeting).ToList();
        }

        public override bool IsReferenced(Meeting item)
        {
            return false;
        }

        #endregion Methods
    }
}