﻿namespace Phoenix.Presenter.Interfaces
{
    using System;

    using Phoenix.Helpers.Misc;
    using Phoenix.Presenter.Helpers;

    /// <summary>
    /// Provides basic features to an insert box. That's a box with
    /// insertion and update.
    /// </summary>
    /// <typeparam name="T">The type of the item this insert box manages.</typeparam>
    public interface ISimpleInsertBox<T>
    {
        #region Events

        /// <summary>
        /// Occurs when user wants to insert an item.
        /// </summary>
        event EventHandler<EventArgs<T>> Inserted;

        /// <summary>
        /// Occurs when user wants to update an item.
        /// </summary>
        event EventHandler<EventArgs<T>> Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the item this manager manages.
        /// </summary>
        /// <value>The managed item.</value>
        T ItemToManage
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the usage of this manager.
        /// </summary>
        /// <value>The usage.</value>
        Usage Usage
        {
            get;
            set;
        }

        #endregion Properties
    }
}