﻿namespace Phoenix.Presenter.Interfaces.Manager
{
    using System;
    using System.Collections.Generic;

    using Phoenix.Model.Bll.Domain;

    #region Enumerations

    /// <summary>
    /// Indicates where the validation failed
    /// </summary>d
    public enum MedicalCardTypeFailedOn
    {
        EmptyTitle,
    }

    #endregion Enumerations

    public interface IMedicalCardTypeInputView
    {
        #region Events

        /// <summary>
        /// Occurs when a medical card type is deleted.
        /// </summary>
        event EventHandler Deleted;

        /// <summary>
        /// Occurs when a medical card type is added.
        /// </summary>
        event EventHandler Inserted;

        /// <summary>
        /// Occurs when medical card type isupdated.
        /// </summary>
        event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        MedicalCardType ItemToManage
        {
            get;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears previous error on validation failed.
        /// </summary>
        void ClearError();

        /// <summary>
        /// Sets an error message to explain why data validation failed.
        /// </summary>
        /// <param name="failed">The fail.</param>
        /// <param name="message">The error message.</param>
        void Invalidate(MedicalCardTypeFailedOn failure, string errorMessage);

        #endregion Methods
    }

    public interface IMedicalCardTypeManagerPresenter : IMedicalCardTypeInputView
    {
        #region Events

        /// <summary>
        /// Occurs when the view is refreshed.
        /// </summary>
        event EventHandler Refreshed;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        new MedicalCardType ItemToManage
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets all the medical cards.
        /// </summary>
        /// <value>The medical cards.</value>
        List<MedicalCardType> MedicalCardTypes
        {
            get;
            set;
        }

        #endregion Properties
    }
}