﻿namespace Phoenix.View.Frame.Configuration
{
    using System.Windows.Forms;

    using Phoenix.View.Configuration;

    /// <summary>
    /// Provides a control to manage the shortcutsNode of Phoenix
    /// </summary>
    public class ShortcutConfigurationControl : PropertyGrid, IConfigurationControl
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ShortcutConfigurationControl"/> class.
        /// </summary>
        public ShortcutConfigurationControl()
        {
            this.Shortcuts = new ShortcutConfiguration();
            this.RefreshData();
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the shortcuts.
        /// </summary>
        /// <value>The shortcuts.</value>
        public ShortcutConfiguration Shortcuts
        {
            get; private set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Rollback theconfiguration date to the previous state.
        /// </summary>
        public void Rollback()
        {
            this.Shortcuts.Rollback();
        }

        /// <summary>
        /// Saves the configuration in the config file.
        /// </summary>
        public void SaveConfiguration()
        {
            this.Shortcuts.CommitChanges();
            this.RefreshData();
        }

        private void RefreshData()
        {
            this.Shortcuts.Load();
            base.SelectedObject = this.Shortcuts;
        }

        #endregion Methods
    }
}