﻿namespace Phoenix.View.Input
{
    using Phoenix.Contracts;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;
    using Phoenix.View.Helpers;

    /// <summary>
    /// Manage the doctor types
    /// </summary>
    public class DoctorTypeInput : BasicInput, IDoctorTypeInputPresenter
    {
        #region Fields

        private const string VALUE1 = "Title";
        private const string VALUE2 = "Remark";

        private DoctorTypeModel presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DoctorTypeInput"/> class.
        /// </summary>
        public DoctorTypeInput()
            : base(new DoctorType(), VALUE1, VALUE2)
        {
            this.presenter = new DoctorTypeModel(this);
            this.Title = "Manage doctor type type".Translate();
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public DoctorType ItemToManage
        {
            get { return this.Item as DoctorType; }
            set
            {
                var temp = value as DoctorType;
                this.Bind(temp, VALUE1, VALUE2);
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Sets an error message to explain why data validation failed.
        /// </summary>
        /// <param name="fail">The fail.</param>
        /// <param name="message">The message.</param>
        public void SetError(DoctorTypeFailOn fail, string message)
        {
            switch (fail)
            {
                case DoctorTypeFailOn.Title:
                    this.SetError(On.Value1, message);
                    break;
                case DoctorTypeFailOn.Referenced:
                    PredifinedMsgBoxes.WarnOnReferencedItem();
                    break;
                default:
                    Assert.FailsOnEnumeration(fail);
                    break;
            }
        }

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns>
        /// Returns <c>true</c> if the data is valid; otherwise <b>false</b>
        /// </returns>
        protected override bool ValidateData()
        {
            return this.presenter.ValidateData();
        }

        #endregion Methods
    }
}