﻿namespace Phoenix.View.Management
{
    using System;

    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.View.Helpers;

    /// <summary>
    /// Delete patients
    /// </summary>
    public class PatientDeleter : CaptchaBox, IDeleter<Patient>
    {
        #region Fields

        private PatientDeleterModel model;
        private Patient patient = new Patient();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PatientDeleter"/> class.
        /// </summary>
        public PatientDeleter()
        {
            this.model = new PatientDeleterModel(this);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when the user asked a deletion of an item.
        /// </summary>
        public event EventHandler<EventArgs<Patient>> Deleted;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the item to delete.
        /// </summary>
        /// <value>The item to delete.</value>
        public Patient ItemToDelete
        {
            get
            {
                return this.patient;
            }
            set
            {
                this.patient = value;
                this.ConfirmationFailed += (sender, e) => MessageBoxes.Exclamation("The code is incorrect!");
                this.ConfirmationSucceeded += (sender, e) => this.OnDeleted(patient);
            }
        }

        #endregion Properties

        #region Methods

        private void OnDeleted(Patient patient)
        {
            if (patient != null)
                this.Deleted.Raise(this, new EventArgs<Patient>(patient));
        }

        #endregion Methods
    }
}