﻿namespace Phoenix.Presenter.Search.Criterion
{
    using System.ComponentModel;
    using System.Windows.Forms;

    using Phoenix.Translation;

    /// <summary>
    /// Provides an input for combobox criterion.
    /// </summary>
    public class ComboBoxCriterion : CriterionControl, ICriterionControl
    {
        #region Fields

        private ComboBox cb_Combobox = new ComboBox();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ComboBoxCriterion"/> class.
        /// </summary>
        public ComboBoxCriterion()
        {
            this.cb_Combobox.DropDownStyle = ComboBoxStyle.DropDownList;
            base.AddInCriterionControl(this.cb_Combobox);
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the data source of the combobox.
        /// </summary>
        /// <value>The data source.</value>
        public object DataSource
        {
            get
            {
                return this.cb_Combobox.DataSource;
            }
            set
            {
                this.cb_Combobox.DataSource = value;
            }
        }

        /// <summary>
        /// Gets or sets the display member.
        /// </summary>
        /// <value>The display member.</value>
        public string DisplayMember
        {
            get
            {
                return this.cb_Combobox.DisplayMember;
            }
            set
            {
                this.cb_Combobox.DisplayMember = value;
            }
        }

        /// <summary>
        /// Gets the criterion's number.
        /// </summary>
        /// <value></value>
        /// <number>The criterion's number.</number>
        [Browsable(false)]
        public override object Value
        {
            get
            {
                return this.cb_Combobox.SelectedItem;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Checks the values the user entered and provides
        /// warning message if values are not correct.
        /// </summary>
        /// <returns>
        /// 	<c>True</c> if the check succeed. Otherwise <c>False</c>.
        /// </returns>
        public override bool ValidateCriterions()
        {
            if (this.cb_Combobox.SelectedItem == null)
            {
                string msg = "You didn't select anything!".Translate();
                base.SetError(msg);
                return false;
            }
            else
            {
                return true;
            }
        }

        /// <summary>
        /// Clears the value of the criterion.
        /// </summary>
        protected override void ClearValue()
        {
            if (this.cb_Combobox.DataSource == null)
                this.cb_Combobox.Items.Clear();
        }

        #endregion Methods
    }
}