﻿namespace Phoenix.Helpers.Winforms.Logging
{
    using System;
    using System.IO;
    using System.Windows.Forms;

    public class TextBoxConsoleRedirection : TextWriter
    {
        #region Fields

        private TextBox textBox;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TextBoxConsoleRedirection"/> class.
        /// </summary>
        /// <param name="textBox">The text box.</param>
        public TextBoxConsoleRedirection(TextBox textBox)
        {
            this.textBox = textBox;
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// When overridden in a derived class, returns the <see cref="T:System.Text.Encoding"/> in which the output is written.
        /// </summary>
        /// <value></value>
        /// <returns>
        /// The Encoding in which the output is written.
        /// </returns>
        public override System.Text.Encoding Encoding
        {
            get { return System.Text.Encoding.UTF8; }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Writes a character to the text stream.
        /// </summary>
        /// <param name="value">The character to write to the text stream.</param>
        /// <exception cref="T:System.ObjectDisposedException">
        /// The <see cref="T:System.IO.TextWriter"/> is closed.
        /// </exception>
        /// <exception cref="T:System.IO.IOException">
        /// An I/O error occurs.
        /// </exception>
        public override void Write(char value)
        {
            if (this.textBox.InvokeRequired)
            {
                this.textBox.Invoke((Action)delegate { this.Write(value); });
            }
            else
            {
                base.Write(value);
                if (!this.textBox.IsDisposed)
                    this.textBox.AppendText(value.ToString());
            }
        }

        #endregion Methods
    }
}