﻿namespace Phoenix.S2P.Helpers
{
    using System;
    using System.Collections.Generic;
    using System.Configuration;
    using System.Data;
    using System.Data.OleDb;
    using System.Linq;
    using System.Windows.Forms;

    using log4net;

    using NHibernate;

    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;

    public class Convertor
    {
        #region Fields

        private const string CONN_STRING_ACE = "Provider=Microsoft.ACE.OLEDB.12.0;Data Source={0};";
        private const string CONN_STRING_JET = "Provider=Microsoft.Jet.OLEDB.4.0; Data Source={0};";

        private Practice defaultPractice = new Practice();
        private DoctorType defaultSpecialisation;
        private DoctorType doctor;
        private DoctorType homeopath;
        private MedicalCardType medicalCardType;
        private MeetingType meetingType;
        private User user;

        #endregion Fields

        #region Constructors

        public Convertor(string dbPath, Type type)
        {
            string cstring = string.Empty;
            switch (type)
            {
                case Type.ACE:
                    cstring = string.Format(CONN_STRING_ACE, dbPath);
                    break;
                case Type.JET:
                    cstring = string.Format(CONN_STRING_JET, dbPath);
                    break;
                default:
                    throw new NotSupportedException(string.Format(
                        "The enumeration value '{0}' is not supported!", type));
            }
            this.Connection = new OleDbConnection(cstring);
            this.Logger = LogManager.GetLogger(typeof(Convertor));
            this.Logger.InfoFormat("Connection string: {0}", cstring);

            this.doctor = new DoctorType();
            this.doctor.Title = "Médecin traitant";

            this.homeopath = new DoctorType();
            this.homeopath.Title = "Homéopath";

            this.meetingType = new MeetingType();
            this.meetingType.Title = "Rendez-vous médical";
            this.meetingType.Remark = "Done by Sharpenda2Phoenix DB translator";

            this.medicalCardType = new MedicalCardType();
            this.medicalCardType.Title = "Fiche médicale";

            DbInsert dbInsert = (DbInsert)ConfigurationManager.GetSection("DBInsert");

            this.defaultPractice = dbInsert.Practice;
            this.user = dbInsert.User;
            this.user.IsDefault = true;

            this.defaultSpecialisation = dbInsert.DefaultSpecialisation;
            this.homeopath = dbInsert.Homeopath;
            this.doctor = dbInsert.Doctor;
        }

        #endregion Constructors

        #region Enumerations

        public enum Type
        {
            ACE,
            JET,
        }

        private enum Speciality
        {
            Homeopath,
            Doctor,
        }

        #endregion Enumerations

        #region Events

        public event EventHandler ErrorOccured;

        public event EventHandler Finished;

        #endregion Events

        #region Properties

        public List<Tuple<Reputation>> Reputations
        {
            get;
            set;
        }

        public ISession Session
        {
            get;
            set;
        }

        private OleDbConnection Connection
        {
            get;
            set;
        }

        private List<Tuple<Doctor>> Doctors
        {
            get;
            set;
        }

        private List<Tuple<Drug>> Drugs
        {
            get;
            set;
        }

        private List<Tuple<DrugType>> DrugTypes
        {
            get;
            set;
        }

        private List<Tuple<Insurance>> Insurances
        {
            get;
            set;
        }

        private ILog Logger
        {
            get;
            set;
        }

        private List<Tuple<WorkMeeting>> Meetings
        {
            get;
            set;
        }

        private List<Tuple<Patient>> Patients
        {
            get;
            set;
        }

        private List<Tuple<Picture>> Pictures
        {
            get;
            set;
        }

        private List<Tuple<Prescription>> Prescriptions
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        public void Compute()
        {
            var globalReport = Benchmark.GetReport();
            var readReport = Benchmark.GetReport();
            var writeReport = Benchmark.GetReport();

            using (new Benchmark(globalReport))
            {
                try
                {
                    this.ConfigureDB();
                    this.Connection.Open();

                    using (new Benchmark(readReport))
                    {
                        this.LoadReputations();
                        this.LoadInsurances();
                        this.LoadPatients();
                        this.LoadMeetings();
                        this.LoadDrugTypes();
                        this.LoadDrugs();
                        this.LoadPrescriptions();
                        this.LoadPictures();
                    }

                    using (new Benchmark(writeReport))
                    {
                        this.SaveAll();
                    }

                    this.Finished.Raise(this);
                }
                catch (Exception ex)
                {
                    this.Logger.Fatal(ex);
                    this.ErrorOccured.Raise(this);
                }
                finally
                {
                    this.Connection.Close();
                }
            }
            this.Logger.InfoFormat("Execution time: {0:0.000} sec - Read time: {1:0.000} - Write time: {2:0.000}"
                , globalReport.ElapsedSeconds
                , readReport.ElapsedSeconds
                , writeReport.ElapsedSeconds);
        }

        private void AddAditionalMCard(DataRow row, Patient thisPatient, string suffix)
        {
            if (!row["FICHE_SUPPL_" + suffix].Equals(DBNull.Value)
                && !string.IsNullOrEmpty((string)row["FICHE_SUPPL_" + suffix]))
            {
                MedicalCard mCard = new MedicalCard();
                mCard.Title = "Fiche médicale supplémentaire " + suffix;
                mCard.Remark = this.ConvertIntoRtf((string)row["FICHE_SUPPL_" + suffix]);

                mCard.Type = this.medicalCardType;
                if (!row["FICHE_SUPPL_" + suffix].Equals(DBNull.Value))
                {
                    mCard.CreationDate
                        = mCard.UpdateDate
                        = (DateTime)row["Derniere_MAJ"];
                }
                else
                {
                    mCard.CreationDate
                        = mCard.UpdateDate
                        = DateTime.Today;
                }
                thisPatient.Add(mCard);
            }
        }

        private void AddMedicalCard(DataRow row, Patient thisPatient)
        {
            if (!row["FicheMedicale"].Equals(DBNull.Value)
                && !string.IsNullOrEmpty((string)row["FicheMedicale"]))
            {
                MedicalCard mCard = new MedicalCard();
                mCard.Title = "Fiche médicale";
                mCard.Remark = this.ConvertIntoRtf((string)row["FicheMedicale"]);
                mCard.Type = this.medicalCardType;
                if (!row["FicheMedicale"].Equals(DBNull.Value))
                {
                    mCard.CreationDate
                        = mCard.UpdateDate
                        = (DateTime)row["Derniere_MAJ"];
                }
                else
                {
                    mCard.CreationDate
                        = mCard.UpdateDate
                        = DateTime.Today;
                }
                thisPatient.Add(mCard);
            }
        }

        private void ConfigureDB()
        {
            Logger.Debug("Configuring Phoenix database");

            NHibernate.Cfg.Configuration configuration = new NHibernate.Cfg.Configuration();
            configuration.Configure().AddAssembly("Phoenix.Model.DAL");
            /*
             * If you have an exception complaining about an SQLite Driver not found
             * and you are running x64 OS, set the compilation of the project to 32bits
             */
            ISessionFactory factory = configuration.BuildSessionFactory();
            this.Session = factory.OpenSession();
        }

        private string ConvertIntoRtf(string text)
        {
            RichTextBox converter = new RichTextBox();
            converter.Text = text;
            return converter.Rtf;
        }

        private DataSet ExecuteQuery(string query)
        {
            DataSet dataSet = new DataSet();
            string SELECT_DOCTORS = query;

            OleDbCommand command = new OleDbCommand(SELECT_DOCTORS, this.Connection);
            OleDbDataAdapter adapter = new OleDbDataAdapter(command);

            adapter.Fill(dataSet, "items");
            return dataSet;
        }

        private Doctor GetDoctor(long id)
        {
            if (this.Doctors == null)
                this.LoadDoctors();

            return (from Tuple<Doctor> doctor in this.Doctors
                    where doctor.ID == id
                    select doctor).FirstOrDefault().Value;
        }

        private Drug GetDrug(long id)
        {
            if (this.Drugs == null)
                this.LoadDrugs();

            return (from Tuple<Drug> drug in this.Drugs
                    where drug.ID == id
                    select drug).FirstOrDefault().Value;
        }

        private DrugType GetDrugType(long id)
        {
            if (this.DrugTypes == null)
                this.LoadDrugTypes();

            return (from Tuple<DrugType> type in this.DrugTypes
                    where type.ID == id
                    select type).FirstOrDefault().Value;
        }

        private Insurance GetInsurance(long id)
        {
            if (this.Insurances == null)
                this.LoadInsurances();

            return (from Tuple<Insurance> insurance in this.Insurances
                    where insurance.ID == id
                    select insurance).FirstOrDefault().Value;
        }

        private Patient GetPatient(long id)
        {
            if (this.Patients == null)
                this.LoadPatients();

            return (from Tuple<Patient> patient in this.Patients
                    where patient.ID == (id)
                    select patient).FirstOrDefault().Value;
        }

        private Reputation GetReputations(long id)
        {
            if (this.Reputations == null)
                this.LoadReputations();

            var result = (from Tuple<Reputation> reputation in this.Reputations
                          where reputation.ID == id
                          select reputation).FirstOrDefault().Value;
            return result;
        }

        private void LoadDoctors()
        {
            List<Tuple<Doctor>> result = new List<Tuple<Doctor>>();

            DataSet dataSet = this.ExecuteQuery("SELECT * FROM Medecin;");
            this.Logger.InfoFormat("Loaded {0} doctors", dataSet.Tables[0].Rows.Count);

            foreach (DataRow row in dataSet.Tables[0].Rows)
            {
                Doctor thisDoctor = new Doctor();
                long id = (long)((int)row["id_medecin"]);

                if (!row["Prenom"].Equals(DBNull.Value))
                    thisDoctor.FirstName = (string)row["Prenom"];

                if (!row["Nom"].Equals(DBNull.Value))
                    thisDoctor.LastName = ((string)row["Nom"]).ToUpper();

                Address address = new Address();
                if (!row["Adresse"].Equals(DBNull.Value))
                    address.Street = (string)row["Adresse"];

                if (!row["Ville"].Equals(DBNull.Value))
                    address.City = (string)row["Ville"];

                if (!row["CodePostal"].Equals(DBNull.Value))
                    address.PostalCode = (string)row["CodePostal"];

                thisDoctor.Address = address;

                if (!row["Telephone"].Equals(DBNull.Value))
                    thisDoctor.PhonePro = (string)row["Telephone"];

                if (!row["Fax"].Equals(DBNull.Value))
                    thisDoctor.MobilePro = (string)row["Fax"];

                bool isRegularDoctor = (bool)row["MedecinTraitant"];
                bool isHomeoptath = (bool)row["Homeopathe"];

                if (isRegularDoctor)
                    thisDoctor.Specialisation = this.doctor;
                else if (isHomeoptath)
                    thisDoctor.Specialisation = this.homeopath;
                else
                    thisDoctor.Specialisation = this.defaultSpecialisation;

                result.Add(new Tuple<Doctor>(id, thisDoctor));
            }
            this.Doctors = result;
        }

        private void LoadDrugs()
        {
            DataSet dataSet = this.ExecuteQuery("SELECT * FROM Medicament");
            this.Logger.InfoFormat("Loaded {0} drugs", dataSet.Tables[0].Rows.Count);

            List<Tuple<Drug>> result = new List<Tuple<Drug>>();
            foreach (DataRow row in dataSet.Tables[0].Rows)
            {
                Drug thisDrug = new Drug();
                long id = (long)((int)row["id_Medicament"]);

                if (!row["Rmq"].Equals(DBNull.Value))
                    thisDrug.Title = (string)row["Nom"];

                if (!row["Nom"].Equals(DBNull.Value))
                    thisDrug.Remark = (string)row["Rmq"];

                if (!row["id_Type"].Equals(DBNull.Value))
                    thisDrug.Type = GetDrugType((int)row["id_Type"]);

                result.Add(new Tuple<Drug>(id, thisDrug));
            }

            this.Drugs = result;
        }

        private void LoadDrugTypes()
        {
            DataSet dataSet = this.ExecuteQuery("SELECT * FROM Type_Medicament");
            this.Logger.InfoFormat("Loaded {0} drug types", dataSet.Tables[0].Rows.Count);

            List<Tuple<DrugType>> result = new List<Tuple<DrugType>>();
            foreach (DataRow row in dataSet.Tables[0].Rows)
            {
                DrugType thisDrugType = new DrugType();
                long id = (long)((int)row["id_Type"]);

                if (!row["Type"].Equals(DBNull.Value))
                    thisDrugType.Title = (string)row["Type"];

                if (!row["Rmq"].Equals(DBNull.Value))
                    thisDrugType.Remark = (string)row["Rmq"];

                result.Add(new Tuple<DrugType>(id, thisDrugType));
            }

            this.DrugTypes = result;
        }

        private void LoadInsurances()
        {
            DataSet dataSet = this.ExecuteQuery("SELECT * FROM Honoraire;");
            this.Logger.InfoFormat("Loaded {0} insurances", dataSet.Tables[0].Rows.Count);

            List<Tuple<Insurance>> result = new List<Tuple<Insurance>>();
            foreach (DataRow row in dataSet.Tables[0].Rows)
            {
                Insurance thisInsurance = new Insurance();
                long id = (long)((int)row["id_Honoraire"]);

                if (!row["Type"].Equals(DBNull.Value))
                {
                    string title = (string)row["Type"];
                    thisInsurance.Title = string.IsNullOrEmpty(title) ? "N.A." : title;
                }

                result.Add(new Tuple<Insurance>(id, thisInsurance));
            }
            this.Insurances = result;
        }

        private void LoadMeetings()
        {
            List<Tuple<WorkMeeting>> result = new List<Tuple<WorkMeeting>>();
            DataSet dataSet = this.ExecuteQuery("SELECT * FROM RendezVous;");
            this.Logger.InfoFormat("Loaded {0} meetings", dataSet.Tables[0].Rows.Count);

            foreach (DataRow row in dataSet.Tables[0].Rows)
            {
                WorkMeeting thisMeeting = new WorkMeeting();
                long id = (long)((int)row["id_Rdv"]);
                thisMeeting.Type = this.meetingType;

                if (!row["came_to_rdv"].Equals(DBNull.Value))
                    thisMeeting.PatientCame = (bool)row["came_to_rdv"];

                if (!row["a_paye"].Equals(DBNull.Value))
                    thisMeeting.IsPayed = (bool)row["a_paye"];

                if (!row["Debut"].Equals(DBNull.Value))
                    thisMeeting.StartDate = (DateTime)row["Debut"];

                if (!row["Fin"].Equals(DBNull.Value))
                    thisMeeting.EndDate = (DateTime)row["Fin"];

                if (!row["Titre"].Equals(DBNull.Value))
                    thisMeeting.Title = (string)row["Titre"];

                if (!row["Message"].Equals(DBNull.Value))
                    thisMeeting.Remark = (string)row["Message"];

                Patient patient = this.GetPatient((int)row["id_Patient"]);
                patient.Add(thisMeeting);

                result.Add(new Tuple<WorkMeeting>(id, thisMeeting));
            }

            this.Meetings = result;
        }

        private void LoadPatients()
        {
            DataSet dataSet = this.ExecuteQuery("SELECT * FROM PATIENT");
            this.Logger.InfoFormat("Loaded {0} patients", dataSet.Tables[0].Rows.Count);

            List<Tuple<Patient>> patients = new List<Tuple<Patient>>();
            foreach (DataRow row in dataSet.Tables[0].Rows)
            {
                Patient thisPatient = new Patient();
                long id = (long)((int)row["id_patient"]);
                thisPatient.LastName = (string)row["Nom"];
                thisPatient.FirstName = (string)row["Prenom"];
                thisPatient.PlaceOfBirth = (string)row["Lieu_Naissance"];
                thisPatient.DateOfBirth = (DateTime)row["DateNaissance"];

                Address thisAddress = new Address();
                thisAddress.Street = (string)row["Adresse"];
                thisAddress.City = (string)row["Ville"];
                thisAddress.PostalCode = (string)row["Code_Postal"];

                thisPatient.Address = thisAddress;
                thisPatient.PhonePrivate = (string)row["Telephone_Prive"];
                thisPatient.PhonePro = (string)row["Telephone_Professionel"];
                thisPatient.Sex = ((string)row["Sexe"]).ToUpper();
                thisPatient.Mail = (string)row["Mail"];
                thisPatient.InscriptionDate = (DateTime)row["Prem_Consult"];
                thisPatient.Reason = (string)row["Motif_1"] + (string)row["Motif_2"];

                thisPatient.Reputation = (!row["id_Reputation"].Equals(DBNull.Value))
                    ? this.GetReputations((long)((int)row["id_Reputation"]))
                    : null;

                thisPatient.Insurance = (!row["id_Honoraire"].Equals(DBNull.Value))
                    ? this.GetInsurance((long)((int)row["id_Honoraire"]))
                    : null;

                if (!row["id_Medecin"].Equals(DBNull.Value))
                    thisPatient.Add(this.GetDoctor((int)row["id_Medecin"]));

                if (!row["id_Homeopathe"].Equals(DBNull.Value))
                    thisPatient.Add(this.GetDoctor((int)row["id_Homeopathe"]));

                object lastUpdate = row["Derniere_MAJ"];
                thisPatient.LastUpdate = (lastUpdate == null || lastUpdate.Equals(DBNull.Value))
                    ? (DateTime)row["Prem_Consult"]
                    : (DateTime)row["Derniere_MAJ"];

                thisPatient.Practice = this.defaultPractice;

                this.AddMedicalCard(row, thisPatient);
                this.AddAditionalMCard(row, thisPatient, "1");
                this.AddAditionalMCard(row, thisPatient, "2");
                this.AddAditionalMCard(row, thisPatient, "3");
                this.AddAditionalMCard(row, thisPatient, "4");

                patients.Add(new Tuple<Patient>(id, thisPatient));

            }
            this.Patients = patients;
        }

        private void LoadPictures()
        {
            List<Tuple<Picture>> result = new List<Tuple<Picture>>();
            DataSet dataSet = this.ExecuteQuery("SELECT * FROM Photo;");
            this.Logger.InfoFormat("Loaded {0} pictures", dataSet.Tables[0].Rows.Count);

            foreach (DataRow row in dataSet.Tables[0].Rows)
            {
                Picture thisPicture = new Picture();
                long id = (long)((int)row["id_Photo"]);

                if (!row["Remarques"].Equals(DBNull.Value))
                    thisPicture.Remark = (string)row["Remarques"];

                if (!row["BinPic"].Equals(DBNull.Value))
                    thisPicture.Bitmap = (byte[])row["BinPic"];

                if (!row["Thumb"].Equals(DBNull.Value))
                    thisPicture.Bitmap = (byte[])row["Thumb"];

                Patient patient = this.GetPatient((int)row["id_Patient"]);
                patient.Add(thisPicture);
                result.Add(new Tuple<Picture>(id, thisPicture));
            }
            this.Pictures = result;
        }

        private void LoadPrescriptions()
        {
            DataSet dataSet = this.ExecuteQuery("SELECT * FROM Prescription");
            this.Logger.InfoFormat("Loaded {0} prescriptions", dataSet.Tables[0].Rows.Count);

            List<Tuple<Prescription>> result = new List<Tuple<Prescription>>();
            foreach (DataRow row in dataSet.Tables[0].Rows)
            {
                Prescription thisPrescription = new Prescription();
                long id = (long)((int)row["id_Prescription"]);

                if (!row["Date_P"].Equals(DBNull.Value))
                    thisPrescription.CreationDate = (DateTime)row["Date_P"];

                if (!row["Rmq"].Equals(DBNull.Value))
                    thisPrescription.Remark = (string)row["Rmq"];

                thisPrescription.Drugs.Add(this.GetDrug((int)row["id_Medicament"]));

                Patient patient = this.GetPatient((int)row["id_Patient"]);
                patient.Add(thisPrescription);

                result.Add(new Tuple<Prescription>(id, thisPrescription));
            }

            this.Prescriptions = result;
        }

        private void LoadReputations()
        {
            List<Tuple<Reputation>> result = new List<Tuple<Reputation>>();
            DataSet dataSet = this.ExecuteQuery("SELECT * FROM REPUTATION");
            this.Logger.InfoFormat("Loaded {0} reputations", dataSet.Tables[0].Rows.Count);
            this.Connection.Close();

            foreach (DataRow row in dataSet.Tables[0].Rows)
            {
                Reputation thisReputation = new Reputation();
                long id = (long)((int)row["id_Reputation"]);

                string label = (string)row["Label"];
                thisReputation.Title = string.IsNullOrEmpty(label) ? "N.A." : label;

                result.Add(new Tuple<Reputation>(id, thisReputation));
            }

            this.Reputations = result;
        }

        private void SaveAll()
        {
            this.Logger.Debug("Start saving transaction...");

            this.Session.Transaction.Begin();

            this.Session.Save(this.user);

            foreach (var tuple in this.Doctors) this.Session.Save(tuple.Value);
            this.Logger.Info("Doctors done...");

            foreach (var tuple in this.Insurances) this.Session.Save(tuple.Value);
            this.Logger.Info("Insurances done...");

            foreach (var tuple in this.Reputations) this.Session.Save(tuple.Value);
            this.Logger.Info("Reputations done...");

            foreach (var tuple in this.DrugTypes) this.Session.Save(tuple.Value);
            this.Logger.Info("DrugTypes done...");

            foreach (var tuple in this.Drugs) this.Session.Save(tuple.Value);
            this.Logger.Info("Drugs done...");

            foreach (var tuple in this.Prescriptions) this.Session.Save(tuple.Value);
            this.Logger.Info("Prescriptions done...");

            foreach (var tuple in this.Meetings) this.Session.Save(tuple.Value);
            this.Logger.Info("Meetings done...");

            foreach (var tuple in this.Patients) this.Session.Save(tuple.Value);
            this.Logger.Info("Patients done...");

            this.Session.Transaction.Commit();

            this.Logger.Debug("Transaction commited");
        }

        #endregion Methods
    }

    public class Tuple<T>
    {
        #region Constructors

        public Tuple(long id, T item)
        {
            this.Value = item;
            this.ID = id;
        }

        #endregion Constructors

        #region Properties

        public long ID
        {
            get;
            set;
        }

        public T Value
        {
            get;
            set;
        }

        #endregion Properties
    }
}