﻿namespace Phoenix.S2P
{
    using System;
    using System.Diagnostics;
    using System.IO;
    using System.Threading;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Helpers.Winforms;
    using Phoenix.Helpers.Winforms.Logging;
    using Phoenix.S2P.Helpers;

    public partial class MainForm : Form
    {
        #region Fields

        private const int ELAPSED_TIME = 10;
        private const string SELECTED_DB = "Database: {0}";
        private const string TIME_STATUS = "Elapsed time: {0,2} sec {1,3} msec.";

        private string databasePath = string.Empty;
        private ILog logger = LogManager.GetLogger(typeof(MainForm));
        private Thread thread;
        private System.Windows.Forms.Timer timer = new System.Windows.Forms.Timer();
        private TimeSpan timeSpan = new TimeSpan(0, 0, 0, 0, 0);
        private Convertor.Type type;

        #endregion Fields

        #region Constructors

        public MainForm()
        {
            InitializeComponent();

            this.lbl_SelectedDb.Text = string.Format(SELECTED_DB, string.Empty);
            BasicTextBoxAppender.Configure(this.tb_Logs);
        }

        #endregion Constructors

        #region Methods

        public void ProgressStyle(ProgressBarStyle style)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { this.ProgressStyle(style); });
            }
            else
            {
                this.tspb_Progress.Style = style;
            }
        }

        private void BtnProcessEnabled(bool enabled)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { this.BtnProcessEnabled(enabled); });
            }
            else
            {
                this.btn_Process.Enabled = enabled;
            }
        }

        private void btn_Process_Click(object sender, EventArgs e)
        {
            try
            {
                this.Clean();

                Convertor convertor = new Convertor(this.databasePath, type);

                Action action = (Action)delegate
                {
                    try
                    {
                        this.ProgressStyle(ProgressBarStyle.Marquee);
                        this.BtnProcessEnabled(false);
                        convertor.Compute();
                        this.ProgressStyle(ProgressBarStyle.Blocks);
                    }
                    catch (Exception ex)
                    {
                        this.HandleError(ex);
                        throw;
                    }
                };
                this.thread = new Thread(new ThreadStart(action));
                this.thread.Start();
            }
            catch (Exception ex)
            {
                this.logger.Warn(ex);
                throw;
            }
        }

        private void Clean()
        {
            FileInfo fi = new FileInfo("Database.db");
            if (fi.Exists)
            {
                fi.Delete();
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
        }

        private void HandleError(Exception ex)
        {
            this.logger.Error(ex);
        }

        private void SelectDB()
        {
            DialogResult dr = this.ofd_Database.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.btn_Process.Enabled = true;
            this.databasePath = this.ofd_Database.FileName;
            this.lbl_SelectedDb.Text = string.Format(SELECTED_DB, this.databasePath);
            this.tb_Logs.Text = string.Empty;
        }

        private void tsmi_ACEDatabase_Click(object sender, EventArgs e)
        {
            this.type = Convertor.Type.ACE;
            SelectDB();
        }

        private void tsmi_CopyTo_Click(object sender, EventArgs e)
        {
            try
            {
                var dr = this.fbd_CopyTo.ShowDialog();
                if (dr != DialogResult.OK) return;

                File.Move("Database.db", Path.Combine(this.fbd_CopyTo.SelectedPath, "Database.db"));
                MessageBoxes.Information("Copy executed.");
            }
            catch (Exception ex)
            {
                MessageBoxes.Error("Impossible to copy the DB");
                this.logger.Error(ex);
            }
        }

        private void tsmi_JETDatabase_Click(object sender, EventArgs e)
        {
            this.type = Convertor.Type.JET;
            SelectDB();
        }

        private void tsmi_OpenFolder_Click(object sender, EventArgs e)
        {
            Process.Start(Environment.CurrentDirectory);
        }

        #endregion Methods
    }
}